################################################################################
# Created       : Martin Foster
# Modified      : 19-Mar-2007
################################################################################
#
# Database - Modules part of Ethereal Realms, designed to aid database 
#            access, querying and programming  
# Copyright (C) 2000-2007  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 4-3526 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

package Ethereal::Database;					# Package name

use DB_File;							# Berkely DB addition
use DBI;							# Database indepedent
use strict;							# Strict variable enforcement

################################################################################
# Data Members
################################################################################

################################################################################
# CORE
#
# Core sub-routines are those that are required to be used in order to get basic
# functionality.  Such as connect, the constructor and the destructor.

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data members
        my $class = shift;
        my $self;


	#####################
	# Program area
	
	# Pre-define certain characteristics
        $self = {
	 ADMN         => 'false',				# Administrative forced legacy
	 AUTH         => 'false',				# Is user authenticated
	 OVER         => 'false',				# Legacy override
	 MULTI        => 1,					# Multiple schemas
	 DBHASH       => '/home/erealms/ethereal/db/hash',	# Path for system tags
	 DBROOM       => '/home/erealms/ethereal/db/room',	# Path for room tags
	 TMPL         => 'template',				# Identifier for templates
	 TMPLDB       => undef,					# Template hash database
	 ROOMDB       => undef,					# Room hash database
	 SEARCH       => undef,					# Search path
	 DOMAIN       => undef, 				# Domain name captured
	 TITLE        => undef,					# Title defined
	 REFUSE       => 'http://www.erealms.org/503.shtml',	# Resource unavailable
	 COMP_VANITY  => undef,					# Enable Vanity/Social Networking
	 COMP_HOME    => undef,					# Enable homepages
	 COMP_GALLERY => undef,					# Enable galleries
	 SYS          => {},					# System hash embeded into script
	 USR          => {},					# User authentication details
	 HASH         => []					# Empty array
	 							# containing all created
								# hashes
        };

	# Bless $self into designated class
	bless $self, $class;

	# Return self	
        return $self;
}


#####################
# Database Connect
#
# Sub-routine to allow connection to the database while revealing as little as
# possible to the coder.  Also centralized

sub Connect
{
	#####################
	# Data members

	my $self     = shift;					# Self reference
	my $cgi      = shift;					# Ability to orient
	my $database;				 		# Database handle

	# User variables
	my $dbname	= 'erealms_ethereal';			# Database
	my $host	= 'localhost';				# Server to connect
	my $port	= '5432';				# Port to use
	my $user	= 'erealms_ethereal';			# Login name
	my $pass	= '0bddS5gaMdpNI';			# Password
	
	# Orientation
	my $domain;						# Domain name
	my $search;						# Search path
	my $timeout;						# Timeout on queries

	#####################
	# Program area

	# Establish connection
	$database = DBI->connect("DBI:Pg:dbname=$dbname; user=$user; password=$pass"
		) or $self->ErrExit($cgi, "Database error \#$DBI::err :  $DBI::errstr");

	# Default behaviour
	$database->{AutoCommit} = 0;
	$database->{RaiseError} = 1;

	# Store handle
	$self->{'HANDLE'} = $database;


	# Timeout on queries
	$timeout = ($self->{'ADMN'} eq 'true') ? 0 : 5000;
	

	# Adjust statement timeout
	$database->do("SET SESSION statement_timeout=?", {}, $timeout);
	

	# Orient yourself
	# Check for need
	if ($self->{'MULTI'} != 0)
	{
		# Scripts of with varying callers
		if (defined($cgi))
		{
			# Active search
			$domain =  $cgi->url();
			$domain =~ s/http\:\/\/(\w+\.)?([A-Za-z0-9\-\_]+)\.(\w{2,})\/.+/$2\.$3/si;
	
			# Set search path
			($search) = $self->DataGet("SELECT public.getSearch(?)", lc($domain))
		          if (defined($domain));
		}

		# Generally reserved to commandline
		else
		{
			# Fetch default path only
			($search) = $self->DataGet("SELECT public.getSearch()")
			  unless (defined($search));
		}

		# Store search path	
		$self->{'SEARCH'} = $search;
		$self->{'DOMAIN'} = $domain;
	}

	# Set domain name
	else
	{
		# Ability exists	
		if (defined($cgi))
		{
			# Cut out name
			$domain =  $cgi->url();
			$domain =~ s/http\:\/\/(\w+\.)?([A-Za-z0-9\-\_]+)\.(\w{2,})\/.+/$2\.$3/si;

			# Set entry
			$self->{'DOMAIN'} = $domain;
			$self->{'SEARCH'} = 'erealms';
		}
	}

	# Commit changes
	$self->Commit();


	#
	# Handling of system variables
	$self->GetHashSystem($self->{'SYS'});

	# Establish local overrides if available
	# Vanity/Social networking component
	$self->{'COMP_VANITY'}  = lc($self->{'SYS'}{'SetComponentVanity'})
	  unless (defined($self->{'COMP_VANITY'})); 

	# Home page component
	$self->{'COMP_HOME'}    = lc($self->{'SYS'}{'SetComponentHome'})
	  unless (defined($self->{'COMP_HOME'})); 

	# Image/Text gallery component
	$self->{'COMP_GALLERY'} = lc($self->{'SYS'}{'SetComponentGallery'})
	  unless (defined($self->{'COMP_GALLERY'})); 

	# Single Signon systems
	$self->{'DEFAULT'} = undef
	  if (lc($self->{'SYS'}{'SetSingleSignon'}) eq 'true');
}


#####################
# Destructor
#
# Subroutine that will kick in once the object itself has passed away.  This
# will allow to automatically kill the database connection and this free
# us of adding that code

sub DESTROY
{
	#####################
	# Data Members

	my $self     = shift;					# Reference to self
	
	my $database = $self->{'HANDLE'};			# Database handle
	my $hash     = $self->{'HASH'};				# Hash of ties
	my $linked;						# Linked hash returned
	
	#####################
	# Program area

	# Safety
	# Only bother if something is there
	if (defined($database))
	{
		# Commit to remove any shared locks
		$database->commit();

		# Connection pooling correction
		$database->{'AutoCommit'} = 1;

		# Disconnect from database
		$database->disconnect();


		# Untie any held values
		# Loop through
		while ($linked = shift(@{$hash}))
		{
			# Unlink and move on
			untie $linked;
		}

		# Untie template cache
		untie $self->{'TMPLDB'}
		 if (defined($self->{'TMPLDB'}));

		# Untie room cache
		untie $self->{'ROOMDB'}
		 if (defined($self->{'ROOMDB'}));
	}
}


#####################
# Commit
#
# A smple wrapper for the DBI commit method.

sub Commit
{
	#####################
	# Data Members

	my $self     = shift;					# Reference to self
	my $data     = $self->{'HANDLE'};			# Database handle

	
	#####################
	# Program area

	# Commit changes
	$data->commit();
}


#####################
# Error
#
# Subroutine designed to make things easier for the programmer while
# reducing the change of a script termination.

sub ErrDisplay
{
	#####################
	# Data Members

	my $self     = shift;					# Reference to self
	my $str      = shift;					# String passed to process
	my $err      = shift;					# Error string

	my $index;						# Index number
	my $handle;						# Unique parameter
	my $place;						# Replaced value

	
	#####################
	# Program area

	# Cycle through if necessary
	while ($handle = shift(@_))
	{
		# Handling of NULLs
		$place = (defined($handle)) ? $handle : 'NULL'; 
		$place = (($place =~ /^\d+$/) || ($place eq 'NULL')) ? $place : "'" . $place . "'";

		# Fetch index
		$index = index($str, '?');

		# Employ
		substr($str, $index, 1, $place);
	}

	# Sound error
	die("Transaction Error:\n$err\n\nQuery on $self->{SEARCH}:\n$str\n");
}

#####################
# Error Exit
#
# Slightly more intelligent way of dying in that a proper error code
# will be returned instead.

sub ErrExit
{
	#####################
	# Data Members

	my $self     = shift;					# Reference to self
	my $cgi      = shift;					# CGI Handle
	my $err      = shift;					# Error string


	#####################
	# Program area

	# Die gracefully
	if (defined($cgi))
	{
		# Return error code
		print $cgi->redirect($self->{'REFUSE'});

		# Premature exit
		exit;
	}

	# Simply die
	die($err);
}


#####################
# Rollback
#
# A smple wrapper for the DBI rollback method.

sub Rollback
{
	#####################
	# Data Members

	my $self     = shift;					# Reference to self
	my $data     = $self->{'HANDLE'};			# Database handle

	
	#####################
	# Program area

	# Rollback changes
	$data->rollback();
}


################################################################################
# STATIC
#
# Static sub-routines exist to retreive very specific information based on a few
# parameters passed to it.   This essentially means that it does most of the
# work and already has the queries built in.


#####################
# Default Puppet
#
# While this makes use of Data Get, the use of this function is very much
# static.

sub DataGetDefault
{ 
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $user     = shift;					# Users login name
	my $realm    = shift;					# Realm name
	my $access   = shift;					# Access control

	my $def_pup;						# Default puppet 
	my $def_realm;						# Default realm


	#####################
	# Program area

	# Puppets
	# Non canon handles
	unless ((defined($access)) && ($access eq 'restricted'))
	{
		# Realm pool
		$realm = 'system';

		# Selected default
		($def_pup) = $self->DataGet("SELECT PuppetName
			FROM Puppet
			WHERE PuppetDefault=?
			  AND PuppeteerLogin=?", 'yes', $user);

		# No default specified
		unless (defined($def_pup))
		{
			# Alphebetical search
			($def_pup) = $self->DataGet("SELECT PuppetName 
				FROM Puppet 
				WHERE PuppetPool=? 
				  AND PuppeteerLogin=? 
				ORDER BY PuppetName LIMIT 1",  $realm, $user); 
		}
	}

	# Canon handles
	else
	{
		# Alphebetical search
		($def_pup) = $self->DataGet("SELECT PuppetName
			FROM Puppet
			WHERE PuppetPool=?
			AND PuppeteerLogin=?
			ORDER BY PuppetName LIMIT 1", $realm, $user);
	}


	# Puppeteer
	# Initial pull
	($def_realm) = $self->DataGet("SELECT PuppeteerRealm
		FROM Puppeteer
		WHERE PuppeteerLogin=?", $user);

	# Does not exist
	unless (defined($def_realm))
	{
		# Auxilary default
		($def_realm) = $self->DataGet("SELECT RealmName
			FROM Realm
			WHERE RealmPublic=?
			ORDER BY RealmName LIMIT 1", 'yes');
	}


	# Returns default handle
	return $def_pup, $def_realm;
}


#####################
# Work Average
#
# This allows the gallery to make proper calculations on the ratings of specific
# works and designs.

sub DataGetWorkAvg
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $gid      = shift;					# Gallery ID
	my $wid      = shift;					# WorkID

	my $avg;						# Average
	my $vote;						# Votes


	#####################
	# Program area

	# Retrieve
	($avg, $vote) = $self->DataGet("SELECT
		  ROUND(AVG(CommentRating)),
		  COUNT(CommentRating)
		FROM Comment
		WHERE GalleryID=?
		AND WorkID=?
		AND CommentRating <> 0", 

	 $gid,
	 $wid);

	# Ensure assignment
	$avg  = (defined($avg))  ? $avg  : 0;
	$vote = (defined($vote)) ? $vote : 0;

	# Retrun
	return $avg, $vote;
}


#####################
# Group List
#
# Group list exists to generate a list containing the groups and puppets
# for in chat use.

sub GroupList
{
        #####################
        # Data members
	my $self     = shift;					# Self referencing link
	my $list     = shift;					# Reference to array
	my $access   = shift;					# Access method
	my $user     = shift;					# User name
	my $room     = shift;					# Room name
	my $snub     = shift;					# Ostracize puppets
	my $puppet   = shift;					# Puppet selected

	my $statement;						# Statement handle

	my $current;						# Current group
	my $previous = 'NIL';					# Previous group

	my $cut;						# Cut down on groups
	my $plan;						# Plan of execution


        #####################
	# Program Area

	# What to do
	$plan = ($access ne 'restricted') ? "(PuppetPool='system' OR PuppetPool=?)" : "PuppetPool=?";

	# Ostracize
	if ((defined($snub))
	 && (defined($puppet))
         && ($snub eq 'yes'))
 	{
		# Pull association
		($cut) = $self->DataGet("SELECT PuppetAssociation
			FROM Puppet
			WHERE PuppeteerLogin=?
			  AND PuppetName=?",

		 $user,
		 $puppet
		);

		# Format string
		$cut = (defined($cut)) ? " AND PuppetAssociation='$cut'" : " AND PuppetAssociation IS NULL";
	}

	# Default
	else
	{
		# No value associated
		$cut = '';
	}

	
	# Get things started
	$self->Pull(\$statement, "SELECT
		  PuppetAssociation,
		  PuppetName
		FROM Puppet
		WHERE PuppeteerLogin=?
		  AND PuppetActive IS NULL
		  AND $plan $cut
		ORDER BY PuppetAssociation, LOWER(PuppetName)",
	 $user,
	 $room
	);

	# Loop and analyse
	# Groups planned
	while (($current, $puppet) = $statement->fetchrow())
	{
		# Checks
		# Leave if not needed
		last unless (defined($previous));
		last unless (defined($current));

		# Handling
		# Titles
		if ($current ne $previous)
		{
			# Add a space
			push(@{$list}, '') if ($previous ne 'NIL');

			# Add title
			push(@{$list}, "-- $current --");

			# Make them match
			$previous = $current;
		}
		
		# General insert
		push(@{$list}, $puppet);
	}

	# Verify if there is still something there
	if (defined($puppet))
	{
		# Additional check
		push(@{$list}, '') if ((defined($previous)) && ($previous ne 'NIL'));	

		# Loop and fetch the rest
		do
		{
			# Just put on the rest of the list
			push(@{$list}, $puppet);
		} while (($current, $puppet) = $statement->fetchrow());
	}

	# Finish query
	$statement->finish();
}


#####################
# Document Get
#
# Data retreival sub-routine for tag templates

sub DocumentGet	
{
	#####################
	# Data members

	my $self     = shift;					# Referene to self
	my $field    = shift;					# Search field

	my $data;						# Pulled information
	
	my $admin    = $self->{'ADMN'};				# Administrative control
	my $cache    = $self->{'TMPLDB'};			# Template hash
	my $path     = $self->{'DBHASH'};			# Location of local cache
	my $search   = $self->{'SEARCH'};			# Search path
	my $type     = $self->{'TMPL'};				# Type of tag


	#####################
	# Program area

	# Determine method
	# BerkelyDB
	if (($admin eq 'false')
	 && (defined($path))
	 && (!defined($cache)))
 	{
		# Modify path
		$path .= '/' . $search . '.' . $type; 

		# Additional check
		if (-f $path)
		{
			# Defaults
			my $def = new DB_File::BTREEINFO;
		
			# Load up hash
			tie(%{$cache}, 'DB_File', $path, O_RDONLY, 0666, $def)
			 or die('Unable to link to $type hash: ', $!);

		 	# Assign hash for next pass
			$self->{'TMPLDB'} = $cache;
		}
	}

	# Cache up and running
	if (defined($cache))
	{
		# Simply pull directly
		return $cache->{$field};
	}
	
	# Legacy
	# Initialize and execute
	($data) = $self->DataGet("SELECT TagData 
		FROM  Tag 
		WHERE TagName=?
		  AND TagType=?",

	 $field,
	 $type
	);

	# Return gathered information
	return $data;
}
 
# Alternate Headers
# Alias for Administrative header
sub DocumentGetAdminHeader
{ return DocumentGet(shift, 'AdminHeader'); } 

# Alias for Administrative footer
sub DocumentGetAdminFooter
{ return DocumentGet(shift, 'AdminFooter'); } 
 
# Alias for Contributor header
sub DocumentGetContribHeader
{ return DocumentGet(shift, 'ContribHeader'); } 

# Alias for Contributor footer
sub DocumentGetContribFooter
{ return DocumentGet(shift, 'ContribFooter'); } 

# Alias for Emailer
sub DocumentGetEmailer
{ return DocumentGet(shift, 'MailEmailer'); }

# Alias for Gallery Header
sub DocumentGetGalleryHeader
{ return DocumentGet(shift, 'GalleryHeader'); }

# Alias for Gallery Footer
sub DocumentGetGalleryFooter
{ return DocumentGet(shift, 'GalleryFooter'); }

# Alias for Supervisory header
sub DocumentGetSuperHeader
{ return DocumentGet(shift, 'SuperHeader'); } 

# Alias for Supervisory footer
sub DocumentGetSuperFooter
{ return DocumentGet(shift, 'SuperFooter'); } 

# Board admin
sub DocumentGetBoardAdmin
{ return DocumentGet(shift, 'BoardAdmin'); }

# Board notice
sub DocumentGetBoardNotice
{ return DocumentGet(shift, 'BoardNotice'); }

# Canon handle realm
sub DocumentGetCanon
{ return DocumentGet(shift, 'WebCanon'); }

# Contributor portal
sub DocumentGetContribPortal
{ return DocumentGet(shift, 'ContribPortal'); }

# Donations section
sub DocumentGetDonations
{ return DocumentGet(shift, 'WebDonation'); }

# Donations Mail
sub DocumentGetDonationsMail
{ return DocumentGet(shift, 'MailDonation'); }

# Alias for Gallery Chapter
sub DocumentGetGalleryChapter
{ return DocumentGet(shift, 'GalleryChapter'); }

# Alias for Gallery Export
sub DocumentGetGalleryCover
{ return DocumentGet(shift, 'GalleryCover'); }

# Alias for Gallery Exif
sub DocumentGetGalleryExif
{ return DocumentGet(shift, 'GalleryExif'); }

# Alias for Gallery Export
sub DocumentGetGalleryExport
{ return DocumentGet(shift, 'GalleryExport'); }

# Alias for Gallery Notice
sub DocumentGetGalleryNotice
{ return DocumentGet(shift, 'GalleryNotice'); }

# Alias for Header
sub DocumentGetHeader
{ return DocumentGet(shift, 'WebHeader'); }

# Alias for Header
sub DocumentGetHeaderPlain
{ return DocumentGet(shift, 'WebHeaderPlain'); }

# Alias for Home Header
sub DocumentGetHomeHeader
{ return DocumentGet(shift, 'HomeHeader'); }

# Alias for Home Readme
sub DocumentGetHomeReadme
{ return DocumentGet(shift, 'HomeReadme'); }

# Alias for Footer
sub DocumentGetFooter
{ return DocumentGet(shift, 'WebFooter'); }

# Alias for Invitation
sub DocumentGetInvitation
{ return DocumentGet(shift, 'MailInvitation'); }

# Mail banned
sub DocumentGetMailBanned
{ return DocumentGet(shift, 'MailBanned'); }

# Mail gallery
sub DocumentGetMailGallery
{ return DocumentGet(shift, 'MailGallery'); }

# Mail notice
sub DocumentGetMailNotice
{ return DocumentGet(shift, 'MailNotice'); }

# Mail offline notice
sub DocumentGetMailOffline
{ return DocumentGet(shift, 'MailOffline'); }

# Mail posts notificiation and instructions
sub DocumentGetMailPosts
{ return DocumentGet(shift, 'MailPosts'); }

# Mail confirmed
sub DocumentGetConfirmed
{ return DocumentGet(shift, 'MailVerified'); }

# Alias for Description handler
sub DocumentGetDescription
{ return DocumentGet(shift, 'WebDescription'); }

# Alias for Javascript
sub DocumentGetJavascript
{ return DocumentGet(shift, 'WebJavascript'); }

# Alias for notices
sub DocumentGetNotice
{ return DocumentGet(shift, 'WebNotice'); }

# Alias for Portal
sub DocumentGetPortal
{ return DocumentGet(shift, 'WebPortal'); }

# Alias for Stream Kill
sub DocumentGetReset
{ return DocumentGet(shift, 'MailReset'); }

# Alias for Stream Kill
sub DocumentGetStream
{ return DocumentGet(shift, 'WebStream'); }

# Alias for Webring
sub DocumentGetWebring
{ return DocumentGet(shift, 'WebRing'); }


#####################
# Fetch Realm
#
# Used to retrieve the information from realms, making first sure that they
# have indeed made use of the functionality.  Otherwise things, will tend
# to falter.

sub FetchRealm
{
	#####################
	# Data members

	my $self     = shift;					# Referene to self
	my $field    = shift;					# Search field
	my $realm    = shift;					# Affected realm

	my $value;						# Value pulled from database
	my $room;						# Formatted room name

	my $admin    = $self->{'ADMN'};				# Administrative control
	my $cache    = $self->{'ROOMDB'};			# Template hash
	my $path     = $self->{'DBROOM'};			# Location of local cache
	my $search   = $self->{'SEARCH'};			# Search path


	#####################
	# Program area

	# Determine method
	# BerkelyDB
	if (($admin eq 'false')
	 && (defined($path))
	 && (!defined($cache)))
 	{
		# Room name
		$room =  lc($realm);
		$room =~ s/\W//gs;
		
		# Modify path
		$path .= '/' . $search . '.' . lc($room); 

		# Additional check
		if (-f $path)
		{
			# Defaults
			my $def = new DB_File::BTREEINFO;
		
			# Load up hash
			tie(%{$cache}, 'DB_File', $path, O_RDONLY, 0666, $def)
			 or die('Unable to link to $realm hash: ', $!);

		 	# Assign hash for next pass
			$self->{'ROOMDB'} = $cache;
		}
	}

	# Cache up and running
	if (defined($cache))
	{
		# Simply pull directly
		return $cache->{$field};
	}

	# Legacy method	
	# Initial retrieval
	($value) = $self->DataGetRealmDesign($realm, $field);

	# If no value exists
	# Retreive defaults
	$value = $self->FetchRealm($field, 'system')
	 unless (defined($value));

	# Return value for use
	return $value;
}


# Offline mode
# Alias for offline realms
sub FetchOfflineInfo
{ return FetchRealm(shift, 'OfflineInfo', shift); }

# Alias for offline realms
sub FetchOfflinePage
{ return FetchRealm(shift, 'OfflinePage', shift); }

# Alias for offline realms
sub FetchOfflinePanel
{ return FetchRealm(shift, 'OfflinePanel', shift); }

# Alias for offline realms
sub FetchOfflineSplash
{ return FetchRealm(shift, 'OfflineSplash', shift); }

# Alias for offline realms
sub FetchOfflineWarning
{ return FetchRealm(shift, 'OfflineWarning', shift); }


# Standard active
# Alias for dice roller
sub FetchRealmDice
{ return FetchRealm(shift, 'RealmDice', shift); }

# Alias for footer
sub FetchRealmFooter
{ return FetchRealm(shift, 'RealmFooter', shift); }

# Alias for header 
sub FetchRealmHeader
{ return FetchRealm(shift, 'RealmHeader', shift); }

# Alias for information
sub FetchRealmInfo
{ return FetchRealm(shift, 'RealmInfo', shift); }

# Alias for javascript
sub FetchRealmJavascript
{ return FetchRealm(shift, 'RealmJavascript', shift); }

# Alias for notices
sub FetchRealmNotice
{ return FetchRealm(shift, 'RealmNotice', shift); }

# Alias for panel
sub FetchRealmPanel
{ return FetchRealm(shift, 'RealmPanel', shift); }

# Alias for private messages
sub FetchRealmPrivate
{ return FetchRealm(shift, 'RealmPrivate', shift); }

# Alias for Refresh frame
sub FetchRealmRefresh
{ return FetchRealm(shift, 'RealmRefresh', shift); }

# Alias for auto-scrolling remote control
sub FetchRealmRemote
{ return FetchRealm(shift, 'RealmRemote', shift); }

# Alias for seperator bar
sub FetchRealmSeperator
{ return FetchRealm(shift, 'RealmSeperator', shift); }

# Alias for offline realms
sub FetchRealmSplash
{ return FetchRealm(shift, 'RealmSplash', shift); }

# Alias for supplementary panel code
sub FetchRealmSupp
{ return FetchRealm(shift, 'RealmSupp', shift); }

# Alias for warnings
sub FetchRealmWarning
{ return FetchRealm(shift, 'RealmWarning', shift); }

# Alias for zoom utility
sub FetchRealmZoom
{ return FetchRealm(shift, 'RealmZoom', shift); }


# Frameset features
# Alias for Autoscrolling
sub FetchFrameAuto
{ return FetchRealm(shift, 'RealmFrameAutoscroll', shift); }

# Alias for zoom utility
sub FetchFrameNorm
{ return FetchRealm(shift, 'RealmFrameNormal', shift); }

#####################
# Fetch Banner
#
# This is used to fetch banners for either the system or realm specific.

sub FetchBanner
{ 
	#####################
	# Data members
	my $self     = shift;					# Referene to self
	my $realm    = shift;					# Passed on realm name
	my $tmpl     = shift;					# Passed on file with banners

	my $idnt;						# Identifier
	my $res;						# Fortune data


	#####################
	# Program area

	# Fetch banner
	$tmpl = $self->FetchRealm('RealmBanner', $realm)
	  unless (defined($tmpl));

	
	# Verify for fortune
	if ($tmpl =~ /FRTN([\w\-]+)/s)
	{
		# Force case
		$idnt = lc($1);

		# Retreive data
		($res) = $self->DataGetFortune($idnt);


		# Replace if defined
		$tmpl =~ s/FRTN$1/$res/gs
		  if (defined($res));
	}


	# Return banner
	return $tmpl;
}

# Realm specific banners
sub FetchRealmBanner
{ return FetchBanner(shift, shift); }

# Portal banners
sub FetchGateBanner
{ return FetchBanner(shift, undef, shift); }


#####################
# Realm formatting
#
# This will return a date/time stamp with the formatting as specified
# by the hosts.

sub GetDateFormat
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $realm    = shift;					# Realm name query

	my $format;						# Format style
	my $stamp;						# Actual timestamp

	#####################
	# Program area

	# Pull format
	($format) = $self->DataGet("SELECT RealmDateFormat
		FROM Realm
		WHERE RealmName=?", $realm);

	# Fetch formatted date
	($stamp)  = $self->DataGet("SELECT TO_CHAR(NOW(), ?) AS Realm_Date", $format);


	# Return
	return $stamp;
} 


#####################
# Get Hashes
#
# Sub-routine created to take a reference to an array and retrieve a list of
# all records within a specific field of information.

sub GetHash 
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $hash     = shift;					# Reference of hash
	my $type     = shift;					# Type of values

	my $admin    = $self->{'ADMN'};				# Administrative
	my $database = $self->{'HANDLE'};			# Database handle
	my $file     = $self->{'DBHASH'};			# Path to hashes
	my $search   = $self->{'SEARCH'};			# Search path

	my $key;						# Key of hash
	my $statement;						# SQL Statement
	my $value;						# Value for hash


	#####################
	# Program area

	# More modern use of BerkeleyDB
	# Hashes to cut down on network and db
	# load
	if (($admin eq 'false')
	 && (defined($file)) && (-d $file))
 	{
		# Additional check
		$file .= '/' . $search . '.' . $type;
		
		# Final check
		if (-f $file)
		{
			# Defaults
			my $def = new DB_File::BTREEINFO;

			# Load up hash
			tie(%{$hash}, 'DB_File', $file, O_RDONLY, 0666, $def)
			 or die('Unable to link to $type hash: ', $!);

			# Push reference onto array
			push (@{$self->{'HASH'}}, $hash);


			# Return immediatly
			return 1;
		}
	}

	# Legacy hash creation
	# Prepare and execute
	$statement = $database->prepare("SELECT 
		 TagName, 
		 TagData
		FROM  Tag
		WHERE TagType=?");
	$statement->execute($type);

	# Retreive and use
	while (($key, $value) = $statement->fetchrow())
	{ $hash->{$key} = $value; }

	# Finish statement
	$statement->finish();

	# Return true
	return 1;
}

# Alias for Administrative hash
sub GetHashAdmin
{ GetHash(shift, shift, 'admin'); }

# Alias for Board hash
sub GetHashBoard
{ GetHash(shift, shift, 'board'); }

# Alias for Board hash
sub GetHashBio
{ GetHash(shift, shift, 'bio'); }

# Alias for Configuration hash
sub GetHashConfiguration
{ GetHash(shift, shift, 'configuration'); }

# Alias for Contribution hash
sub GetHashContrib
{ GetHash(shift, shift, 'contrib'); }

# Alias for Error codes
sub GetHashErr
{ GetHash(shift, shift, 'vcodes'); }

# Alias for Frequently Asked Questions
sub GetHashFaq
{ GetHash(shift, shift, 'faq'); }

# Alias for Gallery Hash
sub GetHashGallery
{ GetHash(shift, shift, 'gallery'); }

# Alias for Gummy Hash
sub GetHashGummy
{ GetHash(shift, shift, 'gummy'); }

# Alias for Login hash
sub GetHashLogin
{ GetHash(shift, shift, 'login'); }

# Alias for Mail hash
sub GetHashMail
{ GetHash(shift, shift, 'mail'); }

# Alias for menu hash
sub GetHashMenu
{ GetHash(shift, shift, 'menu'); }

# Alias for Poll hash
sub GetHashPoll
{ GetHash(shift, shift, 'poll'); }

# Alias for Portal hash
sub GetHashPortal
{ GetHash(shift, shift, 'portal'); }

# Alias for Realm hash
sub GetHashRealm
{ GetHash(shift, shift, 'realm'); }

# Alias for System hash
sub GetHashSystem
{ GetHash(shift, shift, 'system'); }

# Alias for user hash
sub GetHashUser
{ GetHash(shift, shift, 'user'); }

# Alias for vanity hash
sub GetHashVanity
{ GetHash(shift, shift, 'vanity'); }


################################################################################
# DYNAMIC
#
# Dynamic sub-routines perform a specific task, such as return a row or a list
# but does not know the formation of the query until it is passed to it.

#####################
# GetList
#
# Sub-routine created to take a reference to an array and retrieve a list of
# all records within a specific field of information.

sub GetList
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $array    = shift;					# Reference of array
	my $query    = shift;					# SQL query

	my $database = $self->{'HANDLE'};			# Database handle
	my $statement;						# SQL Statement


	#####################
	# Program area

	# Evaluate factors
	eval
	{
		# Prepare and execute
		$statement = $database->prepare($query);
		$statement->execute(@_);

		# Cleanup of array
		splice(@{ $array }, 0);

		# Retrieve and place in array
		while ($_ = $statement->fetchrow())
		{ push(@{ $array }, $_); }

		# End statement
		$statement->finish();
	};

	# Sound error
	$self->ErrDisplay($query, $@, @_) if ($@);
}

# Alias for documents
sub GetListDocuments
{
  GetList(shift, shift, 
	"SELECT TagName
	 FROM Tag 
	 WHERE TagType=? 
	 ORDER BY TagName", 'template');
}

# Alias for established genres
sub GetListEstablished
{
  GetList(shift, shift,
	"SELECT GenreName
	 FROM GalleryGenre
	 WHERE GalleryID=?
	 ORDER BY GenreName", @_);
}

# Alias for RSS Feeds
sub GetListFeeds
{
  GetList(shift, shift,
	"SELECT FeedID || ' - ' || FeedTitle
	 FROM Feed
	 WHERE RealmName=?
	 ORDER BY FeedTitle", @_);
}

# Alias for genres
sub GetListGenre
{ 
  GetList(shift, shift, 
	"SELECT GenreName
	 FROM Genre
	 WHERE GenreType=?
           AND GenreSub=?
	 ORDER BY GenreName", @_); 
}

# Alias for genres
sub GetListGroup
{ 
  GetList(shift, shift, 
	"SELECT AssociationName
	 FROM Association
	 WHERE PuppeteerLogin=?
	 ORDER BY AssociationName", @_); 
}

# Alias for public realms
sub GetListPublicRealms
{
  GetList(shift, shift, 
	"SELECT RealmName
	 FROM Realm
	 WHERE RealmPublic=?
	 ORDER BY RealmName", 'yes'); 
}

# Alias for work sort worder
sub GetListWorkSort
{
  GetList(shift, shift,
	"SELECT WorkID
	 FROM Work
	 WHERE GalleryID=?
	   AND WorkID > 1
	 ORDER BY WorkSort", @_); 
}

# Alias for private realms
sub GetListSupervisorRealms
{
  GetList(shift, shift,
	"SELECT RealmName
	 FROM RealmSupervisor
	 WHERE PuppeteerLogin=?
	 ORDER BY RealmName", @_); 
}

# Alias for non-private realms
sub GetListNonPrivate
{
  GetList(shift, shift,
	"SELECT RealmName
	 FROM Realm
	 WHERE (RealmPublic=?
	    OR RealmPublic=?)
	   AND RealmName NOT IN ('system', 'banned')
	 ORDER BY RealmName", 'yes', 'hybrid'); }

# Alias for non-private realms
sub GetListVerifiedRealms
{
  GetList(shift, shift,
	"SELECT RealmName
	 FROM Realm
	 WHERE RealmPublic <> ?
	 AND RealmVerified=?
	 ORDER BY RealmName", 'yes', 'yes'); 
}

# System and Realm handles
sub GetListPuppetRealm
{
  GetList(shift, shift, 
	"SELECT PuppetName
	 FROM Puppet
	 WHERE PuppeteerLogin=?
	   AND PuppetActive IS NULL
	   AND (PuppetPool=?
	    OR PuppetPool=?)
	 ORDER BY PuppetName", shift, 'system', @_);
}

# Alias for complete puppet listing
sub GetListPuppetComplete
{ 
  GetList(shift, shift, 
	"SELECT PuppetName
	 FROM Puppet
	 WHERE PuppeteerLogin=?
	 ORDER BY PuppetName", @_); 
}

# Alias for active puppet listing that are yours
sub GetListPuppetYours
{
  GetList(shift, shift, 
	"SELECT PuppetName
	 FROM Puppet
	 WHERE PuppetPool=?
	   AND PuppeteerLogin=?
	   AND PuppetActive IS NULL
	 ORDER BY PuppetName", 'system', @_); 
}

# Alias for puppet listing that are yours
sub GetListPuppetSuspended
{
  GetList(shift, shift, 
	"SELECT PuppetName
	 FROM Puppet
	 WHERE PuppeteerLogin=?
	 AND PuppetActive IS NOT NULL
	 ORDER BY PuppetName", @_); 
}

# Alias for EMail retreival
sub GetListEMails
{
  GetList(shift, shift, 
	"SELECT p.PuppeteerEMail
	 FROM Puppeteer p,
	      PuppeteerRegistration r
	 WHERE p.PuppeteerLogin <> r.PuppeteerLogin
	 GROUP BY PuppeteerEMail
	 ORDER BY PuppeteerEMail"); 
}


# Canon realm exceptions
# Alias for Canon handles
sub GetListCanonPuppets
{
  GetList(shift, shift, 
	"SELECT PuppetName
	 FROM Puppet
	 WHERE PuppeteerLogin=?
	   AND PuppetActive IS NULL
	   AND PuppetPool=?
	 ORDER BY PuppetName", @_); 
}

# Alias for realm genres
sub GetListRealmGenre
{ 
  GetList(shift, shift, 
	"SELECT GenreName
	 FROM Genre
	 WHERE GenreType='realm'
           AND GenreSub='<root>'
	 ORDER BY GenreName", @_); 
}

# Alias for locale
sub GetListRealmLocale
{
  GetList(shift, shift, 
	"SELECT RealmLocale
	 FROM RealmLocale
	 WHERE RealmName=?
	 ORDER BY RealmLocale", @_); 
}

# Restrictions
# Alias for restricted words
sub GetListWords
{  
  GetList(shift, shift, 
	"SELECT RestrictionValue
	 FROM Restriction
	 WHERE RealmName=?
	   AND RestrictionType=?
	 ORDER BY RestrictionValue", shift, 'word', @_); 
}

# Banned images
sub GetListImageBan
{
  GetList(shift, shift, 
	"SELECT RestrictionValue
	 FROM Restriction
	 WHERE RealmName=?
	   AND RestrictionType=?", 'system', 'picture', @_); 
}

# Board Topics
sub GetListTopics
{
  GetList(shift, shift, 
	"SELECT BoardTopic
	 FROM BoardTopic
	 WHERE RealmName=?
	 ORDER BY BoardTopic", @_); 
}


#####################
# GetData
#
# Generic sub-routine that will return a single row of data be it in multiple
# amounts or just one field within the row there is a special section for
# font handling if choice is at all defined.

sub DataGet
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $query    = shift;					# Database query

	my $database = $self->{'HANDLE'};			# Database get
	my $statement;						# SQL Statement

	my @values;						# Values returned from table


	#####################
	# Program area

	# Evaluate factors
	eval
	{
		# Prepare and execute
		$statement = $database->prepare($query);
		$statement->execute(@_);

		# Retreive and finish
		@values = $statement->fetchrow();
		$statement->finish();
	};

	# Verify for error
	$self->ErrDisplay($query, $@, @_) if ($@);

	# Return retreived information
	return @values;
}

# Authentication
# OpenID Throttling
sub DataGetIPandOpenID
{
  return DataGet(shift, 
	"SELECT COUNT(*)
	 FROM Audit
	 WHERE PuppeteerHostname=?
	   AND AuditCode=? 
	   AND AuditTimeStamp > GetInterval('1 Hour')", shift, 1004);
}

# Boards
# Alias for administrative Email
sub DataGetRealmBoard
{
  return DataGet(shift, 
	"SELECT RealmBoard
	 FROM Realm
	 WHERE RealmName=?",  @_); 
}


# Dates
# Alias for short date
sub DataGetDate
{
  return DataGet(shift, "SELECT getDate(NOW()) AS Short_Date", @_);
}

# Alias for RSS date/time
sub DataGetDateRSS
{
  return DataGet(shift, "SELECT getDateRSS(NOW()) AS RSS_Date", @_);
}

# Alias for full timestamp
sub DataGetDateTime
{
  return DataGet(shift, "SELECT getDateTime(NOW()) AS Long_Date", @_);
}

# Frequency
sub DataGetFrenquency
{
  return DataGet(shift,
	"SELECT COUNT(*)
	 FROM Puppeteer
	 WHERE PuppeteerTimestamp > getInterval('" . shift(@_) . " day')", @_);
}


# Gummies
# Alias for class lookup
sub DataGetGummyClass
{
  return DataGet(shift, 
	"SELECT GummyClass
	 FROM Gummy
	 WHERE GummyPath=?",  @_); 
}

# Alias for name lookup
sub DataGetGummyName
{
  return DataGet(shift, 
	"SELECT GummyName
	 FROM Gummy
 	 WHERE GummyPath=?",  @_); 
}

# Alias for pathname lookup
sub DataGetGummyPath
{
  return DataGet(shift, 
	"SELECT GummyPath
	 FROM Gummy
	 WHERE GummyName=?",  @_); 
}


# Parameter list
sub DataGetParam
{
  return DataGet(shift, 
	"SELECT ParamValue
	 FROM Param
	 WHERE PuppeteerLogin=?
	   AND ParamNumber=?::INT", @_); 
}

# Sequence number
sub DataGetSequence
{
  return DataGet(shift, 
	"SELECT ParamNumber
	 FROM Param
	 WHERE PuppeteerLogin=?
	 AND ParamNumber=?::INT", @_); 
}


# Privileges
# Alias for Administrative  option
sub DataGetAdmin
{
  return DataGet(shift, 
	"SELECT PuppeteerAdminPrivs
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?", @_); 
}

# Alias for Supervisory Option
sub DataGetSuper
{ 
  return DataGet(shift, 
	"SELECT PuppeteerSuperPrivs
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?", @_); 
}

# Alias for Privileges
sub DataGetPrivs
{
  return DataGet(shift, 
	"SELECT PuppeteerSuperPrivs,
	        PuppeteerAdminPrivs,
	        PuppeteerModPrivs, 
	        PuppeteerVerified
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?", @_);
}

# Realm privs
sub DataGetRoomPrivs
{
  return DataGet(shift, 
	"SELECT p.PuppeteerLogin
	 FROM Puppeteer p,
	      RealmSupervisor r
	 WHERE p.PuppeteerLogin=r.PuppeteerLogin
	   AND r.RealmName=?
	   AND p.PuppeteerLogin=?
	   AND p.PuppeteerSuperPrivs=?", shift, shift, 'yes', @_); }

# Moderator counts
sub DataGetModCount
{
  return DataGet(shift, 
	"SELECT COUNT(*)
	 FROM RealmSupervisor
	 WHERE PuppeteerLogin=?
	   AND RealmACL < ?", @_);
}


# Posts
# Post ID Number
sub DataGetPostID
{
  return DataGet(shift, 
	"SELECT PostIDNumber
	 FROM Post
	 WHERE PostTimeStamp > getInterval('10 Minutes') LIMIT 1", @_);
}

# Puppeteer
# Alias for Puppeteer Info
sub DataGetPuppeteerLogin
{
  return DataGet(shift,
	"SELECT PuppeteerLogin
	 FROM Puppet
	 WHERE PuppetPool=?
	   AND PuppetName ILIKE ?", 'system', @_); 
}

# Alias for canon handles
sub DataGetPuppeteerCanon
{ 
  return DataGet(shift, 
	"SELECT PuppeteerLogin
	 FROM Puppet
	 WHERE PuppetPool ILIKE ?
	 AND PuppetName ILIKE ?", @_); 
}

# Alias for contact settings
sub DataGetPuppeteerContact
{ 
  return DataGet(shift, 
	"SELECT PuppeteerContact
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?", @_); 
}

# Alias for font settings
sub DataGetFont
{
	# Pull values
	my ($text, $color, $face) = DataGet(shift, 
		"SELECT PuppetTextSize,
		        PuppetTextColour,
		        PuppetTextFace
		 FROM Puppet
		 WHERE PuppetName=?
		 AND PuppeteerLogin=?", shift, shift); 

	# Verify and define
	$text  = (defined($text))  ? $text  : 3;
	$color = (defined($color)) ? $color : shift;
	$face  = (defined($face))  ? $face  : shift;

	return ($text, $color, $face);
}

# Alias for Javascript option
sub DataGetJavascript
{
  return DataGet(shift,
	"SELECT PuppeteerJavascript
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?", @_); 
}

# Alias for chat options
sub DataGetChatOptions
{
  return DataGet(shift, 
	"SELECT PuppeteerJavascript,
	        PuppeteerChatPref
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?", @_); 
}

# Alias for Formatting option
sub DataGetFormatting
{
  return DataGet(shift, 
	"SELECT PuppeteerFormatting,
                PuppeteerChatPref,
                PuppeteerPrivacy 
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?", @_); 
}

# Alias for being ignored
sub DataGetIgnored
{
  return DataGet(shift, 
	 "SELECT PuppeteerLogin
          FROM PuppetIgnore
          WHERE PuppeteerLogin=?
            AND (PuppetName=?
             OR  PuppetLogin=?)
            AND PuppetIgnore IN ('single','global')", @_);
}

# Alias for Ignore Scope
sub DataGetScope
{
  return DataGet(shift,
	"SELECT PuppeteerIgnore
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?",  @_);
}

# Alias for sbubbing
sub DataGetSnub
{
  return DataGet(shift,
	"SELECT PuppeteerSnub
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?",  @_);
}

# Alias for Supervisory option
sub DataGetBannedStatus
{
  return DataGet(shift, 
	"SELECT PuppeteerLogin
	 FROM PuppeteerRestriction
	 WHERE PuppeteerLogin=?
	   AND RealmName=?", @_); 
}


# Alias for Puppeteers Control over a realm
sub DataGetPuppeteerControl
{
  return DataGet(shift,
	"SELECT RealmACL
	 FROM  RealmSupervisor
	 WHERE RealmName=?
	  AND PuppeteerLogin=?", @_);
}

# Alias for PuppeteerEmail retreival
sub DataGetPuppeteerEmail
{
  return DataGet(shift, 
	"SELECT PuppeteerEmail, PuppeteerMailer
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?", @_); 
}

# Alias for Puppeteers Email
sub DataGetPuppeteerExistence
{
  return DataGet(shift, 
	"SELECT PuppeteerLogin, PuppeteerPassword
	 FROM Puppeteer
	 WHERE PuppeteerLogin ILIKE ?", @_); 
}

# Alias for Puppeteers Homepage Expire Time
sub DataGetPuppeteerExpire
{
  return DataGet(shift, 
	"SELECT getExpire(?)", @_);
}

# Alias for Puppeteers Homepage
sub DataGetPuppeteerHome
{
  return DataGet(shift,
	"SELECT PuppeteerHome
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?", @_); 
}

# Alias for puppeteer leech requests
sub DataGetPuppeteerLeeches
{
  return DataGet(shift,
	"SELECT COUNT(*)
	 FROM Audit 
	 WHERE PuppeteerLogin=? 
	   AND AuditCode=? 
	   AND AuditTimeSTAMP > GetInterval('1 Hour')", shift, 1107); 
}

# Alias for Mail retreival
sub DataGetPuppeteerMailCount
{
  return DataGet(shift, 
	"SELECT COUNT(*)
	 FROM Mail
	 WHERE MailRcptLogin=?", @_); 
}

# Alias for mailer settings
sub DataGetPuppeteerMailer
{
 return DataGet(shift,
	 "SELECT PuppeteerMailer, PuppeteerLogin
	  FROM Puppeteer
	  WHERE PuppeteerEmail ILIKE ?",  @_);
}

# Alias for new Mail retreival
sub DataGetPuppeteerMailNew
{
  return DataGet(shift, 
	"SELECT COUNT(*)
	 FROM Mail
	 WHERE MailRcptLogin=?
	   AND MailRcptRead=?", 'no', @_); 
}

# Alias for Name retreival
sub DataGetPuppeteerName
{
  return DataGet(shift, 
	"SELECT PuppeteerName
	 FROM Puppeteer
	 WHERE PuppeteerLogin ILIKE ?", @_); 
}

# Alias for registration
sub DataGetPuppeteerReg
{ 
  return DataGet(shift, 
	"SELECT PuppeteerLogin
	 FROM PuppeteerRegistration
	 WHERE PuppeteerLogin=?", @_);
}

# Alias for verified
sub DataGetVerified
{
  return DataGet(shift, 
	"SELECT PuppeteerVerified
	 FROM Puppeteer
	 WHERE PuppeteerLogin=?", @_); 
}

# Puppets
# Active
sub DataGetPuppetActive
{
  return DataGet(shift, 
	"SELECT PuppetActive
	 FROM Puppet
	 WHERE PuppetName=?
	   AND PuppeteerLogin=?", @_); 
}

# Currently used puppet
sub DataGetPuppetCurrent
{
  return DataGet(shift,
	"SELECT PuppetName
	 FROM Post
	 WHERE RealmName=?
	   AND PuppeteerLogin=?
	   AND PostTimestamp > GetInterval('10 Minutes')
	 ORDER BY PostTimestamp DESC LIMIT 1", @_);
}

# Creation
sub DataGetPuppetCreate
{
  return DataGet(shift,
	"SELECT PuppetName 
 	 FROM  Puppet 
	 WHERE PuppetName  ILIKE ?
	   AND PuppetShort ILIKE ?
	   AND (PuppeteerLogin=?
	    OR PuppetPool='system')", @_);
}

# Existence
sub DataGetPuppetExistence
{
  return DataGet(shift, 
	"SELECT PuppetName
	 FROM Puppet
	 WHERE PuppetName ILIKE ?
	 AND PuppeteerLogin=?", @_); 
}

# Alias for puppets locale settings
sub DataGetPuppetLocale
{
  return DataGet(shift, 
	"SELECT PuppetLocale
	 FROM Puppet
	 WHERE PuppetName=?
	   AND PuppeteerLogin=?", @_); 
}

# Alias for title and tagline
sub DataGetPuppetFormat
{
  return DataGet(shift,
	"SELECT PuppetTitle,
	        PuppetTag
	 FROM Puppet
	 WHERE PuppetName=? 
	   AND PuppeteerLogin=?", @_); 
}

# Alias for short names
sub DataGetPuppetShort
{
  return DataGet(shift,
	"SELECT PuppeteerLogin,
	 FROM Puppet
	 WHERE PuppetShort=?", @_); 
}

# Alias for puppet updates
sub DataGetPuppetUpdates
{
  return DataGet(shift,
	"SELECT COUNT(*)
	 FROM Audit 
	 WHERE PuppeteerLogin=? 
	   AND AuditCode=? 
	   AND AuditTimeSTAMP > GetInterval('1 Hour')", shift, 1106); 
}


# Realm
# Alias for realm access
sub DataGetRealmAccess
{
  return DataGet(shift, 
	"SELECT RealmAccess,
	        RealmVerified
	 FROM Realm
	 WHERE RealmName=?", @_); 
} 

# Alias for realm access
sub DataGetRealmACL
{
  return DataGet(shift, 
	"SELECT RealmACL
	 FROM RealmSupervisor
	 WHERE Puppeteerlogin=?
	   AND RealmName=?", @_); 
}

# Alias for alternate message boards
sub DataGetRealmAltBoard
{ 
  return DataGet(shift, 
	"SELECT RealmAltBoard
	 FROM Realm
	 WHERE RealmName=?", @_); 
}

# Alias for realm archive address
sub DataGetRealmArchive
{
  return DataGet(shift,
	"SELECT RealmArchive
	 FROM Realm
	 WHERE RealmName=?", @_); 
}

# Alias for realm description
sub DataGetRealmDescription
{
  return DataGet(shift, 
	"SELECT DesignContent
	 FROM RealmDesign
	 WHERE TagName=?
	 AND RealmName=?", 'RealmInfo', @_); 
}

# Alias for realm description
sub DataGetRealmDesign
{
  return DataGet(shift, 
	"SELECT DesignContent
	 FROM  RealmDesign
	 WHERE RealmName=?
	 AND TagName=?", @_);
}

# Alias for Realm existence
sub DataGetRealmExistence
{
  return DataGet(shift,
	"SELECT RealmName,
	        RealmPublic,
	        RealmAlias
	 FROM Realm
	 WHERE RealmName ILIKE ?", @_); 
}

# Alias for realm fonts
sub DataGetRealmFont
{
  return DataGet(shift, 
	"SELECT RealmFontColor,
	        RealmFontFace,
	        RealmFontSize
	 FROM Realm
	 WHERE RealmName=?", @_); }

# Alias for Realm Harassment
sub DataGetRealmHarass
{
  return DataGet(shift, 
	"SELECT RealmHarass
	 FROM Realm
	 WHERE RealmName=?", @_); 
}

# Alias for Realm Image Info
sub DataGetRealmImageInfo
{
  return DataGet(shift, 
	"SELECT RealmImageHeight,
	        RealmImageWidth,
	        RealmImageSize
	 FROM Realm
	 WHERE RealmName=?", @_); 
}

# Alias for locale settings
sub DataGetRealmLocale
{
  return DataGet(shift, 
	"SELECT RealmLocale
	 FROM Realm
	 WHERE RealmName=?", @_); 
}


# Alias for Realm password
sub DataGetRealmPassword
{
  return DataGet(shift, 
	"SELECT RealmPassword
	 FROM Realm
	 WHERE RealmName=?", @_); 
}

# Alias for Realm Public status
sub DataGetRealmPublic
{
  return DataGet(shift,
	 "SELECT RealmPublic,
	         RealmVerified
	 FROM Realm
	 WHERE RealmName=?", @_); 
}

# Alias for Realm Public status
sub DataGetRealmText
{
  return DataGet(shift, 
	"SELECT RealmTextColour
	 FROM Realm
	 WHERE RealmName=?", @_);
}


# Social aspects
# Alias for full locate
sub DataGetSocialLocate
{ 
  return DataGet(shift, 
	"SELECT LocateCountry,
	        LocateRegion,
		LocateCity,
		LocateLat,
		LocateLong
	 FROM Social
	 WHERE PuppeteerLogin=?", @_);
}

# Alias for short locate
sub DataGetSocialShort
{ 
  return DataGet(shift, 
	"SELECT LocateLat,
		LocateLong
	 FROM Social
	 WHERE PuppeteerLogin=?", @_);
}

# Alias for Astrological signs
sub DataGetSocialAstro
{ 
  return DataGet(shift, 
	"SELECT AstroSign
	 FROM Astrology
	 WHERE AstroStart <= ?
	 AND AstroEnd >= ?", '2000-' . $_[0], '2000-' . $_[1]);
}

# Alias for chineese signs
sub DataGetSocialSign
{ 
  return DataGet(shift, 
	"SELECT SignAnimal, SignElement
	 FROM Sign
	 WHERE SignStart <= ?
	 AND SignEnd >= ?", @_);
}

# Alias for Interests
sub DataGetSocialInterests
{ 
  return DataGet(shift, 
	"SELECT SocialInterests
	 FROM Network
	 WHERE PuppeteerLogin=?", @_);
}


# Misc
# Insert ID
sub DataGetInsert
{
  return DataGet(shift,
	"SELECT NEXTVAL(?)", @_);
}


# Weather control
sub DataGetWeather
{
  return DataGet(shift, 
	"SELECT RealmMETAR
	 FROM Realm
	 WHERE RealmName=?", @_); 
}

# Weather control
sub DataGetWeatherICAO
{
  return DataGet(shift, 
	"SELECT RealmMETAR
	 FROM Realm
	 WHERE RealmICAO=?", @_); 
}


# Fortunes
sub DataGetFortune
{
  return DataGet(shift, 
	"SELECT FortuneMsg
	 FROM Fortune
	 WHERE FortuneType=?
	 ORDER BY RANDOM() LIMIT 1", @_);
}

# Alias for font code
sub DataGetFontCode
{
  return DataGet(shift, 
	"SELECT FontCode
	 FROM Font
	 WHERE FontName=?
	  AND FontType=?", @_);
}

# Weblog articles
sub DataGetWeblogEntries
{
  return DataGet(shift,
	 "SELECT COUNT(*), getNew(MAX(LogTimestamp))
	 FROM Weblog", @_);
}


# Social
# Source Liasons
sub DataGetLiason
{
  return DataGet(shift,
	  "SELECT LiasonSource, LiasonRelation
	   FROM Liason
	   WHERE LiasonOwner=?
	     AND LiasonTarget=?", @_);
}

# Search preferences
sub DataGetSearchPrefs
{
  return DataGet(shift,
	  "SELECT PhysicalGender,
	          SocialSeeking,
		  SocialAgeMin,
		  SocialAgeMax, 
		  LocateCity,
		  LocateRegion,
		  LocateCountry,
		  LocateLat,
		  LocateLong
	   FROM Network
	   WHERE PuppeteerLogin=?", @_);
}

# Gallery
# Access privs
sub DataGetGalleryAccess
{
  return DataGet(shift, 
	"SELECT AccessOwner,
       		AccessEditor,
	       	AccessJanitor,
		PuppetName
	 FROM GalleryAccess
	 WHERE GalleryID=?
	   AND PuppeteerLogin=?", @_);
}

# Alternate gallery basics
sub DataGetGalleryBasics
{
  return DataGet(shift, 
	"SELECT GalleryName, GalleryPenName
	 FROM Gallery
	 WHERE GalleryID=?", @_); 
}

# Granted access privs
sub DataGetGalleryGranted
{
  return DataGet(shift, 
	"SELECT COUNT(*)
	 FROM GalleryAccess
	 WHERE PuppeteerLogin=?", @_);
}

# Gallery existence
sub DataGetGalleryID
{
  return DataGet(shift, 
	"SELECT GalleryID
	 FROM Gallery
	 WHERE GalleryName ILIKE ?", @_); 
}

# Janitorial gallery basics
sub DataGetGalleryJanitor
{
  return DataGet(shift, 
	"SELECT PuppeteerLogin
	 FROM GalleryAccess
	 WHERE GalleryID=?
	   AND PuppeteerLogin=?
	   AND (AccessOwner='yes' 
	    OR  AccessJanitor='yes')", @_); 
}

# Gallery layer existence
sub DataGetLayer
{
  return DataGet(shift, 
	"SELECT LayerID, 
	   LayerLevel,
	   LevelZero,
	   LevelOne,
	   LevelTwo,
	   LevelThree,
	   LevelFour,
	   LevelFive
	 FROM Layer
	 WHERE GalleryID=? 
	   AND LayerShort ILIKE ?", @_);
}

# Gallery layer work name
sub DataGetLayerName
{
  return DataGet(shift, 
	"SELECT LayerShort
	 FROM Layer
	 WHERE GalleryID=?
	   AND LayerID=?", @_); 
}

# Gallery layer work name
sub DataGetImageThumbnail
{
  return DataGet(shift, 
	"SELECT WorkID, WorkName, WorkImageThumbnail
	 FROM Work
	 WHERE WorkName ILIKE ?
	   AND GalleryID=?", @_); 
}

# Alternate Gallery existence
sub DataGetGalleryName
{
  return DataGet(shift, 
	"SELECT GalleryName
	 FROM Gallery
	 WHERE GalleryID=?", @_); 
}

# Alternate gallery owner
sub DataGetGalleryOwner
{
  return DataGet(shift, 
	"SELECT PuppeteerLogin
	 FROM Gallery
	 WHERE GalleryID=?", @_); 
}

# Gallery Type
sub DataGetGalleryType
{
  return DataGet(shift, 
	"SELECT GalleryType, GallerySub, GallerySort
	 FROM Gallery
	 WHERE GalleryID=?", @_); 
}

# Gallery Watch
sub DataGetGalleryWatch
{
  return DataGet(shift, 
	"SELECT PuppeteerLogin
	 FROM GalleryNotify
	 WHERE GalleryID=?
	 AND PuppeteerLogin=?", @_); 
}

# Gallery Comments
sub DataGetGalleryComment
{
  return DataGet(shift, 
	"SELECT GalleryComments
	 FROM Gallery
	 WHERE GalleryID=?", @_); 
}

# Work ID
sub DataGetWorkID
{
  return DataGet(shift, 
	"SELECT WorkID
	 FROM Work
	 WHERE WorkName ILIKE ?
	   AND GalleryID=?", @_); 
}

# Work Name
sub DataGetWorkName
{
  return DataGet(shift, 
	"SELECT WorkName, WorkImageThumbnail
	 FROM Work
	 WHERE WorkID=?
	   AND GalleryID=?", @_); 
}

# Work Sort
sub DataGetWorkSort
{
  return DataGet(shift, 
	"SELECT WorkSort
	 FROM Work
	 WHERE WorkID=?
	   AND GalleryID=?", @_); 
}


# Banner and HTML rotation
# Retrieve
sub DataGetRotate
{
  return DataGet(shift,
	"SELECT RotateData, RotateConduct
	 FROM Rotate
	 WHERE RotateName=?
	   AND RotateZone=?", @_);
}


# Vanity related works
# Scores
sub DataGetVanityScores
{
 return DataGet(shift,
	 "SELECT AVG(VoteLevel),
		 COUNT(VoteLevel)
	  FROM VanityVotes
	  WHERE VanityID=?", @_);
}

# Default image
sub DataGetVanityDefault
{
 return DataGet(shift,
	 "SELECT VanityThumb
	  FROM Vanity
	  WHERE PuppeteerLogin=?
	    AND VanityDefault=?", shift, 'yes');
}

# Default image
sub DataGetVanity
{
 return DataGet(shift,
	 "SELECT VanityPath, VanityHeight, VanityWidth
	  FROM Vanity
	  WHERE PuppeteerLogin=?
	    AND VanityDefault=?", shift, 'yes');
}


#
# Web Polls

# Poll details
sub DataGetPoll
{
 return DataGet(shift,
	 "SELECT PollName, PollQuestion
	  FROM Poll 
	  WHERE PollID=?", @_);
}

# Vote has been made
sub DataGetPollVote
{
 return DataGet(shift,
	 "SELECT VoteHostname
 	  FROM PollVote
	  WHERE PollID=?
	   AND VoteHostname=?", @_);
}

# Vote totals
sub DataGetPollTotal
{
 return DataGet(shift,
	 "SELECT COUNT(*)
	  FROM PollVote
	   WHERE PollID=?", @_);
}


#####################
# Show Banner
#
# Very simple wrapper around the banner display which will only invoke the
# call to the database if the user has not yet been in the realm specifically.
# This reduces the load on the database and bandwidth consumption as a result.

sub ShowBanner
{
	####################
	# Data members
	my $self     = shift;					# Reference to self
	my $param    = shift;					# Parameter list

	my $harass;						# Permit hosts to harass

	####################
	# Program Area

	# Check harasshment
	($harass) = $self->DataGetRealmHarass($param->{'ROOM'});

	# First time
	# Or harassment
	if ((!defined($param->{'ROOT'})) 
	 || ($harass eq 'yes'))
	{
		# Display banner
		print $self->FetchRealmBanner($param->{'ROOM'}), "\n";
	}
}


#####################
# Pull
#
# Pull is a wrapper for the complexity behind preparing and retreiving
# information using the prepare and execute handlers.

sub Pull
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $sth      = shift;					# Query handle
	my $query    = shift;					# Database query

	my $capture  = -1;					# Captured return value
	my $database = $self->{'HANDLE'};			# Database handle


	#####################
	# Program area

	# Evaluate measures
	eval
	{
		# Prepare query
		${$sth}  = $database->prepare($query);

		# Execute
		$capture = ${$sth}->execute(@_);
	};

	# Sound error
	if ($@)
	{
		# Display error
		$self->ErrDisplay($query, $@, @_) if ($@);

		# Roll back query
		$database->rollback();

		# Force an undefined retrun
		return undef;
	}

	# Return value
	return $capture;
}


####################
# Quick Write
#
# As the name implies, this sub-routine will write and commit information to
# the database.   This can take place in the form of an update or insert
# statement.  This specific implementation is simple, and is not intended
# for massive transactions.

sub Quick
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $query    = shift;					# Database query

	my $data     = $self->{'HANDLE'};			# Database handle 


	#####################
	# Program area

	# Evaluation block
	eval
	{
		# Run query
		$data->do($query, {}, @_);
      
		# Commit changes
		$data->commit;
 	};

	# If block failed
	if ($@)
	{
		# Issue warning
		$self->ErrDisplay($query, $@, @_);

		# Undo incomplete changes
		$data->rollback;
	}
}


####################
# Write
#
# As the name implies, this sub-routine will write and commit information to
# the database.   This can take place in the form of an update or insert
# statement.  This specific requires an external commit statement.

sub Write
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $query    = shift;					# Database query

	my $data     = $self->{'HANDLE'};			# Database handle 


	#####################
	# Program area

	# Run command
	return $data->do($query, {}, @_);
}

1;								# Return true
